var EXPORTED_SYMBOLS = ["dslrPoller"];

const {classes: Cc, interfaces: Ci, utils: Cu} = Components;

Cu.import("resource://gre/modules/XPCOMUtils.jsm");
Cu.import("resource://gre/modules/Services.jsm");
XPCOMUtils.defineLazyModuleGetter(this, "logger", "chrome://dslrnotifier2/content/modules/logger.jsm");
XPCOMUtils.defineLazyModuleGetter(this, "dslrStatus", "chrome://dslrnotifier2/content/modules/status.jsm");
XPCOMUtils.defineLazyModuleGetter(this, "Prefs", "chrome://dslrnotifier2/content/modules/prefs.jsm");
XPCOMUtils.defineLazyModuleGetter(this, "Privacy", "chrome://dslrnotifier2/content/modules/privacy.jsm");

var dslrPoller = {
	start: function() {
		// Add preference Listener
		Prefs.addListener(Poller);
		
		Poller.init();
	},
	
	stop: function() {
		// Stop timer
		Poller.cancel(true);
	
		// Remove preference Listener
		Prefs.removeListener(Poller);
	},
	
	checkNow: function() {
		Poller.dslrPollIt();
	}
}

var Poller = {
	_timer: null,
	_two_second_timer: null,
	polling: false,
	scaledBack: false,
	errorCount: 0,
	stringBundle: null,
	xmlR: null,
	
	init: function(aDelay){
		var ival = aDelay || Prefs.pollIntMinutes;
		this.cancel(ival == -1);
		if (ival != -1) {
			logger("Timer init with delay of " + ival + " minute(s).");
			var startingTimer = !this._timer;
			if (startingTimer)
				this._timer = Cc["@mozilla.org/timer;1"].createInstance(Ci.nsITimer);
				
			this._timer.init(this, ival*60000, Ci.nsITimer.TYPE_REPEATING_SLACK);
			
			if (startingTimer) {
				logger("Polling timer started");
				
				// Listen for cookie changes (for when logged in/out)
				Services.obs.addObserver(Poller, "cookie-changed", true);
				Services.obs.addObserver(Poller, "private-cookie-changed", true);
				
				// Watch for change in status values
				dslrStatus.addWatch();
				
				// Do first poll in 2 seconds
				this.retry();
			}
		}
	},
	
	retry: function() {
		if (!this._two_second_timer)
			this._two_second_timer = Cc["@mozilla.org/timer;1"].createInstance(Ci.nsITimer);
		else 
			this._two_second_timer.cancel();
			
		this._two_second_timer.init(this, 2000, Ci.nsITimer.TYPE_ONE_SHOT);
	},

	// if aStoppingTimer is false, then timer is being stopped and restarted
	cancel: function(aStoppingTimer){
		if (this._two_second_timer) {
			this._two_second_timer.cancel();
			this._two_second_timer = null;
		}
			
		if (Poller.polling) {
			this.xmlR.abort();
			Poller.polling = false;
		}
	
		if(this._timer){
			logger("Polling timer canceled");
			this._timer.cancel();
			
			if (aStoppingTimer) {
				logger("Polling timer stopped");
				this._timer = null;
			
				// Remove watching for cookies
				Services.obs.removeObserver(Poller, "cookie-changed");
				Services.obs.removeObserver(Poller, "private-cookie-changed");
				
				// Stop watching for change in status values
				dslrStatus.removeWatch();
			
				dslrStatus.reset();
				Services.obs.notifyObservers(null, "DSLRNotifier:status-change", null);
			}
		}
		
		this.xmlR = null;
	},
	
	observe: function(aSubject, aTopic, aData){
		switch (aTopic)
		{
		case "timer-callback":
			logger("Timer callback.");
			Poller.dslrPollIt();	
			break;
		case "nsPref:changed":
			switch(aData) {
				case "pollIntMinutes":
					var ival = Prefs.pollIntMinutes;
					if ((ival == -1) && this._timer) 
						this.cancel(true);
					else if (ival != -1) 
						this.init(ival);
					break;
			}
			break;
		case "cookie-changed":
		case "private-cookie-changed":
			let cookie = aSubject.QueryInterface(Components.interfaces.nsICookie2);
			// host has a "." in front of it so remove it.
			if ((cookie.host.substring(1) == Prefs.siteName) && (cookie.name == "dsl"))
				Poller.dslrPollIt();
			break;
		}
	},
	QueryInterface: XPCOMUtils.generateQI([Ci.nsISupportsWeakReference, Ci.nsIObserver]),

	loaded: function(aEvent) {
		Poller.polling = false;
		Poller.errorCount = 0;
		var xmlR = aEvent.target;
					
		try {
			if (xmlR.status == 200) {
				logger(xmlR.responseText);
				var dslrDoc = xmlR.responseXML;
				var dslrElement = dslrDoc.documentElement;
				var dslrImFlag = dslrElement.getAttribute("i");
				var dslrTopFlag = dslrElement.getAttribute("n");
				var checkTop = Prefs.pollTop;
				var checkIM = Prefs.pollIm;

				dslrStatus.errorFlag = false;
				dslrStatus.repeatingAlert = false;
				
				if ((dslrElement.tagName != "m") && (dslrElement.tagName == "err")) {
					
					dslrStatus.errStr = dslrElement.getAttribute("str");
					dslrStatus.loginFlag = true;
					dslrStatus.imFlag = false;
					dslrStatus.newsFlag = false;
					
					if (Prefs.autoLogin) {
						// Try to grab a non-private window if possible.  If permanent private browsing, allow private windows
						var win = Privacy.getWindow();
																	 
						// If found window, open login tab otherwise 
						if (win && "gDslrNotifier" in win)
							win.gDslrNotifier.dslrAct('tab','login','',true);
						else 
							dslrStatus.showLoginPageOnWindowOpen = true;
					}
				}
				else {
					dslrStatus.loginFlag = false;
					dslrStatus.errStr = "";
				}
						
				if ((dslrImFlag != null) && checkIM) {
					dslrStatus.imText = parseInt(dslrImFlag);
					dslrStatus.imFlag = true;
					dslrStatus.repeatingAlert = (Prefs.repeatImSlide || Prefs.repeatImSound);
				}
				else {
					dslrStatus.imFlag = false;
					dslrStatus.imText= "";
					dslrStatus.impoponce = false;
					dslrStatus.imslideonce = false;
					dslrStatus.imringOnce = false;
				}
						
				if ((dslrTopFlag != null) && checkTop) {
					// Check the offset
					var topOffset = Prefs.feedOff;
					var numTop = parseInt(dslrTopFlag);
					if (!isNaN(numTop) && (numTop > topOffset)) {
						dslrStatus.newsText = parseInt(dslrTopFlag);;
						dslrStatus.newsFlag = true;
						dslrStatus.repeatingAlert = dslrStatus.repeatingAlert || (Prefs.repeatTopSlide || Prefs.repeatTopSound);
					}
					else {
						dslrStatus.newsFlag = false;
						dslrStatus.newsText = "";
						dslrStatus.toppoponce = false;
						dslrStatus.topslideonce = false;
						dslrStatus.topringOnce = false; 
					}
				}
				else {
					dslrStatus.newsFlag = false;
					dslrStatus.newsText = "";
					dslrStatus.toppoponce = false;
					dslrStatus.topslideonce = false;
					dslrStatus.topringOnce = false; 
				}
				
				// If scaled back, reset to user specified interval
				if (Poller.scaledBack) {
					logger("Server has resumed normal opperation, resuming regular checking schedule.", true);
					Poller.scaledBack = false;
					Poller.init();
				}

			}
			else { 
				// Server error
				dslrStatus.errStr = xmlR.statusText;
				dslrStatus.loginFlag = true;
				dslrStatus.errorFlag = true;
				dslrStatus.imFlag = false;
				dslrStatus.newsFlag = false;

				// If server returned error 503 scale back requests to once every 10 minutes
				if (xmlR.status == 503 && !Poller.scaledBack) { 
					logger("Server is busy, scaling back requests to once every 10 minutes.", true);
					Poller.scaledBack = true;
					Poller.init(10);
				}
			}

			// If only the text changed, set the once flags to false to force an alert
			if (dslrStatus.imTextChanged) {
				dslrStatus.imTextChanged = false;
				dslrStatus.impoponce = false;
				dslrStatus.imslideonce = false;
				dslrStatus.imringOnce = false;
			}
			if (dslrStatus.newsTextChanged) {
				dslrStatus.newsTextChanged = false;
				dslrStatus.toppoponce = false;
				dslrStatus.topslideonce = false;
				dslrStatus.topringOnce = false; 
			}
			
			// If anything changed kick off processing
			if (dslrStatus.changed) {
				Services.obs.notifyObservers(null, "DSLRNotifier:status-change", null);
				dslrStatus.changed = false;
			}
			else if (dslrStatus.repeatingAlert)
				Services.obs.notifyObservers(null, "DSLRNotifier:repeat-alert", null);
			
			logger(JSON.stringify(dslrStatus));
		}
		catch (e) { logger(e, true); }
	},
		
	errorHandler: function(aEvent) {
		Poller.polling = false;
		Poller.errorCount++;
		logger("Error " + aEvent.target.status + " occurred while trying to poll server. Error Count = " + Poller.errorCount);

		// If failed 3 times in a row
		if (Poller.errorCount > 2) {
			
			Poller.errorCount = 0;
			try {
				// XMLHttpRequest threw an error
				logger("Error " + aEvent.target.status + " occurred while trying to poll server.");
				
				try {
					if (!this.stringBundle) 
						this.stringBundle = Services.strings.createBundle("chrome://dslrnotifier2/locale/dslrnotifier2.properties");
					dslrStatus.errStr = this.stringBundle.GetStringFromName("poll.error");
				} catch(ex) {
					dslrStatus.errStr = "";
					logger("String Bundle Error: " + ex);
				}
				dslrStatus.loginFlag = true;
				dslrStatus.errorFlag = true;
				dslrStatus.imFlag = false;
				dslrStatus.newsFlag = false;
				
				// If anything changed kick off processing
				if (dslrStatus.changed) {
					Services.obs.notifyObservers(null, "DSLRNotifier:status-change", null);
					dslrStatus.changed = false;
				}
			}
			catch (e) {logger(e, true); }
		}
		else {
			// Try again in two seconds
			Poller.retry();
		}	
	},
		
	dslrPollIt: function() {
		try { 
			if (Poller.polling) {
				logger("Poller called with poll request outstanding");	
				return;
			}
			Poller.polling = true;
				
			logger("Polling now.");
			var sitename = Prefs.siteName;
				
			var pollUrl = "http://www."+sitename+"/browser/extensions/reportstatus.xml?ver=1.1";
			var xmlR = this.xmlR = Cc["@mozilla.org/xmlextras/xmlhttprequest;1"].createInstance(Ci.nsIXMLHttpRequest);

			try {
				// Open connection and fetch data
				xmlR.open("GET", pollUrl, true);
				xmlR.channel.loadFlags |= Ci.nsIRequest.LOAD_BYPASS_CACHE;
				// Force to allow third party cookies (only works in Firefox 3.6 (1.9.2) and above)
				var hci = xmlR.channel.QueryInterface(Ci.nsIHttpChannelInternal);
				if (typeof(hci.forceAllowThirdPartyCookie) == "boolean") {
					hci.forceAllowThirdPartyCookie = true;
				}
				// need to use private channel if permanent private browsing is enabled
				if (Privacy.isPermanentPrivateBrowsing()) {
					let channel = xmlR.channel.QueryInterface(Components.interfaces.nsIPrivateBrowsingChannel);
					channel.setPrivate(true);
				}
				xmlR.setRequestHeader( "Pragma", "no-cache, must-revalidate, proxy-revalidate" );
				xmlR.setRequestHeader( "Cache-Control", "no-cache, must-revalidate, proxy-revalidate" );
				xmlR.overrideMimeType("text/xml");
				xmlR.onload = Poller.loaded;
				xmlR.onerror = Poller.errorHandler;
				// If ready send it, otherwise try again later.
				if (xmlR.readyState == 1) {
					xmlR.send(null);
				}
				else {
					logger("ReadyState = " + xmlR.readyState + ", trying again in two seconds.");
					Poller.polling = false;
					xmlR.abort();
					Poller.retry();
				}
			}
			catch(ex) {
				Poller.polling = false;
				xmlR.abort();
				logger(ex, true); 
			}
		}
		catch (ex) { 
			Poller.polling = false;
			logger(ex, true); 
		}
	}
}

